﻿-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Aloft")
local SML = AceLibrary("SharedMedia-1.0")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Aloft"] = true,

	["HostileNameplatesEnabled"] = "Hostile nameplates |cff00ff00enabled|r.",
	["HostileNameplatesDisabled"] = "Hostile nameplates |cffff0000disabled|r.",
	["FriendlyNameplatesEnabled"] = "Friendly nameplates |cff00ff00enabled|r.",
	["FriendlyNameplatesDisabled"] = "Friendly nameplates |cffff0000disabled|r.",

	["Report Status Changes"] = true,
	["Prints the status of nameplates whenever they change"] = true,

	["Show Nameplates"] = true,
	["Show nameplates for neutral or hostile units"] = true,
	["Show Friendly Nameplates"] = true,
	["Show nameplates for friendly units"] = true,
	
	["Data Options"] = true,
	["Data options"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Aloft"] = "Aloft",

	["HostileNameplatesEnabled"] = "적대적 대상의 이름표 : |cff00ff00사용|r.",
	["HostileNameplatesDisabled"] = "적대적 대상의 이름표 : |cffff0000중지|r.",
	["FriendlyNameplatesEnabled"] = "우호적 대상의 이름표 : |cff00ff00사용|r.",
	["FriendlyNameplatesDisabled"] = "우호적 대상의 이름표 : |cffff0000중지|r.",

	["Report Status Changes"] = "설정 변경사항 알리기",
	["Prints the status of nameplates whenever they change"] = "설정 변경 사항을 대화창에 표시합니다.",

	["Show Nameplates"] = "이름표 보기",
	["Show nameplates for neutral or hostile units"] = "중립적 또는 적대적인 대상의 이름표 보기",
	["Show Friendly Nameplates"] = "우호적인 플레이어의 이름표 보기",
	["Show nameplates for friendly units"] = "우호적인 대상의 이름표 보기",
} end)

-----------------------------------------------------------------------------

Aloft = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0", "AceHook-2.1")

Aloft:RegisterDB("AloftDB")
Aloft:RegisterDefaults('profile', 
	{
		reportStatusChanges = true,
	}
)

-----------------------------------------------------------------------------

Aloft.Options = 
{
	handler = Aloft,
	type = 'group',
	args = 
	{
		show =
		{
			type = 'toggle',
			name = L["Show Nameplates"],
			desc = L["Show nameplates for neutral or hostile units"],
			get = function() return NAMEPLATES_ON end,
			set = function(v) if v then ShowNameplates() else HideNameplates() end NAMEPLATES_ON = v end,
			order = 1
		},
		showFriendly =
		{
			type = 'toggle',
			name = L["Show Friendly Nameplates"],
			desc = L["Show nameplates for friendly units"],
			get = function() return FRIENDNAMEPLATES_ON end,
			set = function(v) if v then ShowFriendNameplates() else HideFriendNameplates() end FRIENDNAMEPLATES_ON = v end,
			order = 2
		},
		reportStatusChanges =
		{
			type = 'toggle',
			name = L["Report Status Changes"],
			desc = L["Prints the status of nameplates whenever they change"],
			get = function() return Aloft.db.profile.reportStatusChanges end,
			set = function(v) Aloft.db.profile.reportStatusChanges = v end,
			order = 3
		},
		moduleSpacer  = 
		{
			type = 'header',
			order = 4,
		},
		dataSpacer = 
		{
			type = 'header',
			order = 500,
		},
		data =
		{
			type = 'group',
			name = L["Data Options"],
			desc = L["Data options"],
			order = 600,
			args = { },
		},
	},
}

Aloft:RegisterChatCommand({"/aloft"}, Aloft.Options)

-----------------------------------------------------------------------------

local numberOfWorldChildren = 0
local nameplateList = { }
local nameToNameplateMap = { }
local visibleNameplateList = { }
local dataSourceList = { }

-----------------------------------------------------------------------------

function Aloft:OnEnable()
	self:ScheduleRepeatingEvent("AloftUpdate", self.Update, 0.2, self)

	-- Without these two lines, the game doesn't show the nameplates on login if they were previously enabled
	if NAMEPLATES_ON then ShowNameplates() end
	if FRIENDNAMEPLATES_ON then ShowFriendNameplates() end
	
	self.HideNameplates = HideNameplates
	self.ShowNameplates = ShowNameplates
	self.HideFriendNameplates = HideFriendNameplates
	self.ShowFriendNameplates = ShowFriendNameplates
	HideNameplates = function() if NAMEPLATES_ON then self:ScheduleEvent("AloftReportNameplateStatus", self.ReportNameplateStatus, 0, self) self.HideNameplates() end end
	ShowNameplates = function() if not NAMEPLATES_ON then self:ScheduleEvent("AloftReportNameplateStatus", self.ReportNameplateStatus, 0, self) self.ShowNameplates() end end
	HideFriendNameplates = function() if FRIENDNAMEPLATES_ON then self:ScheduleEvent("AloftReportNameplateStatus", self.ReportNameplateStatus, 0, self) self.HideFriendNameplates() end end
	ShowFriendNameplates = function() if not FRIENDNAMEPLATES_ON then self:ScheduleEvent("AloftReportNameplateStatus", self.ReportNameplateStatus, 0, self) self.ShowFriendNameplates() end end

	for _, module in self:IterateModules() do
		self:ToggleModuleActive(module, true)
	end

	self:DetermineDataSources()
	
	-- Reinstate hooks
	for aloftData in self:IterateNameplates() do
		self:HookNameplate(aloftData)
	end
end


function Aloft:OnDisable()
	self:CancelScheduledEvent("AloftUpdate")

	HideNameplates = self.HideNameplates
	ShowNameplates = self.ShowNameplates
	HideFriendNameplates = self.HideFriendNameplates
	ShowFriendNameplates = self.ShowFriendNameplates
	
	for _, module in self:IterateModules() do
		self:ToggleModuleActive(module, false)
		module:DisableDataSource()
	end
	
	self:RedisplayNameplates()
	self:RedisplayFriendlyNameplates()
end

function Aloft:ReportNameplateStatus()
	if self.db.profile.reportStatusChanges then
		self:Print((NAMEPLATES_ON and L["HostileNameplatesEnabled"] or L["HostileNameplatesDisabled"]).." "..
					(FRIENDNAMEPLATES_ON and L["FriendlyNameplatesEnabled"] or L["FriendlyNameplatesDisabled"]))
	end
end

function Aloft:IsNameplateFrame(frame)
	if frame:GetName() then
		return false
	end

	local overlayFrame = frame:GetRegions()
	if not overlayFrame or overlayFrame:GetObjectType() ~= "Texture" or overlayFrame:GetTexture() ~= "Interface\\Tooltips\\Nameplate-Border" then
		return false
	end
	
	return true
end

function Aloft:ProcessChildFrames(...)
	for i = 1, select('#', ...) do
		local frame = select(i, ...)
		if not frame.aloftData and self:IsNameplateFrame(frame) then 
			self:SetupNameplate(frame)
			visibleNameplateList[frame.aloftData] = true
		end
	end
end

function Aloft:Update()
	local newNumberOfWorldChildren = WorldFrame:GetNumChildren()
	if newNumberOfWorldChildren ~= numberOfWorldChildren then
		numberOfWorldChildren = newNumberOfWorldChildren

		self:ProcessChildFrames(WorldFrame:GetChildren())
	end
	
	for aloftData in pairs(visibleNameplateList) do	
		local healthBar = aloftData.healthBar
		local r, g, b, a = healthBar:GetStatusBarColor()

		-- An awful awful update routine to stop the game overwriting our colors sporadically
		if r ~= aloftData.healthBarR or g ~= aloftData.healthBarG or b ~= aloftData.healthBarB or a ~= aloftData.healthBarA then
			self:DetermineNameplateType(aloftData, healthBar)
			self:UpdateData(aloftData)
			self:TriggerEvent("Aloft:OnHealthBarColorChanged", aloftData)
		end		
	end
end

-----------------------------------------------------------------------------

function Aloft:IterateNameplates()
	return pairs(nameplateList)
end

function Aloft:IterateVisibleNameplates()
	return pairs(visibleNameplateList)
end

-----------------------------------------------------------------------------

local dataNameToModuleMap = { }
local moduleToModuleListMap = { }

function Aloft:UpdateDataNameToModuleMap(module, ...)
	if not select(1, ...) then return end
	for i = 1,select('#', ...) do
--		ChatFrame3:AddMessage(string.format("Added data module: %s (%s)", tostring(select(i, ...)), tostring(module)))
		dataNameToModuleMap[select(i, ...)] = module
	end
end

function Aloft:UpdateModuleToModuleListMap(module, ...)
	if not select(1, ...) then return end
	
	local moduleList = { }
	moduleToModuleListMap[module] = moduleList
	
	for i = 1,select('#', ...) do
		local dataName = select(i, ...)
		local requiredModule = dataNameToModuleMap[dataName]
--		ChatFrame3:AddMessage(string.format("%s requires %s", tostring(module), dataName))
		if requiredModule then
			moduleList[requiredModule] = true
		else
			self:Print("Unable to determine module providing data: "..dataName)
		end
	end
end

function Aloft:RemoveRequiredModuleFromModuleToModuleListMap(requiredModule)
	for module,moduleList in pairs(moduleToModuleListMap) do
		moduleList[requiredModule] = nil
	end
end

function Aloft:DetermineDataSources()
	-- Empty our table.. we keep this around for IsDataAvailable() calls
	dataNameToModuleMap = { }

	-- 1. Iterate all modules, disable data sources, gather provided data
	for _,module in self:IterateModules() do
		module:DisableDataSource()
		self:UpdateDataNameToModuleMap(module, module:ProvidesData())
	end

	-- Iterate all of the modules for data that they require, resolve them to the modules that provide them
	for name,module in self:IterateModules() do
		self:UpdateModuleToModuleListMap(module, module:RequiresData())
		module:DisableDataSource()
	end

	-- Clear our old data sources
	for i = 1, #dataSourceList do
		dataSourceList[i] = nil
	end

	-- Iterate to remove purley data modules
	local iterationCount = 0	-- A safety factor
	local requiredModules = { }
	while iterationCount < 100 and next(moduleToModuleListMap) do
		for module, moduleList in pairs(moduleToModuleListMap) do
			for requiredModule in pairs(moduleList) do
				requiredModules[requiredModule] = true
			end
		end
		local moduleRemoved = false
		for module, _ in pairs(moduleToModuleListMap) do
			if module:IsPurelyData() and not requiredModules[module] then
--				ChatFrame3:AddMessage("Not required: "..tostring(module))
				moduleToModuleListMap[module] = nil
				self:RemoveRequiredModuleFromModuleToModuleListMap(module)
				moduleRemoved = true
			end
		end
		if not moduleRemoved then break end
		for k in pairs(requiredModules) do
			requiredModules[k] = nil
		end
		iterationCount = iterationCount + 1
	end

	-- Iterate our moduleToModuleListMap 
	iterationCount = 0	-- A safety factor
	while iterationCount < 100 and next(moduleToModuleListMap) do
		for module, moduleList in pairs(moduleToModuleListMap) do
			for requiredModule in pairs(moduleList) do
				if not moduleToModuleListMap[requiredModule] then
					-- The required module isn't dependent on anything! Put it into our dataSourceList and remove from all modules that request it
					table.insert(dataSourceList, requiredModule)
					requiredModule:EnableDataSource()
--					ChatFrame3:AddMessage(#dataSourceList..": "..tostring(requiredModule))

					self:RemoveRequiredModuleFromModuleToModuleListMap(requiredModule)
				end
			end
			if not next(moduleList) then
				moduleToModuleListMap[module] = nil
			end
		end
		iterationCount = iterationCount + 1
	end

	if next(moduleToModuleListMap) then
		self:Print("Internal error: Dependency list not resolved - cyclic dependency?")
		moduleToModuleListMap = { }
	end

	for aloftData in self:IterateVisibleNameplates() do
		self:UpdateData(aloftData)
	end
end

-----------------------------------------------------------------------------

function Aloft:GetTargetNameplate()
	if not UnitExists("target") then return nil end
	
	for aloftData in pairs(visibleNameplateList) do
		if aloftData.nameplateFrame:GetAlpha() == 1 then
			return aloftData
		end
	end
	return nil
end

function Aloft:GetNameplate(unitName)
	return nameToNameplateMap[unitName]
end

function Aloft:GetNameplateFrame(unitName)
	local aloftData = self:GetNameplate(unitName)
	if aloftData then return aloftData.nameplateFrame end
	return nil
end

function Aloft:GetUnitNameplate(unitid)
	if not UnitExists(unitid) then
		return nil 
	end
	return self:GetNameplateFrame(UnitName(unitid))
end

-----------------------------------------------------------------------------

function Aloft:PrepareText(textFrame, data)
	textFrame:SetFont(SML:Fetch("font", data.font), data.fontSize, data.outline)
	if data.shadow then
		textFrame:SetShadowOffset(1, -1)
	else
		textFrame:SetShadowOffset(0, 0)
	end
	if data.alignment then
		textFrame:SetJustifyH(data.alignment)
	end
	if data.color then
		textFrame:SetTextColor(unpack(data.color))
	end
end

function Aloft:PlaceFrame(frame, relativeFrame, data)
	if data.enable then
		frame:ClearAllPoints()
		frame:SetPoint(data.point, relativeFrame, data.relativeToPoint, data.offsetX, data.offsetY)
		
		if data.size then
			frame:SetWidth(data.size)
			frame:SetHeight(data.size)
		else
			if data.width then
				frame:SetWidth(data.width)
			end
			if data.height then
				frame:SetWidth(data.width)
			end
		end
		frame:SetAlpha(data.alpha or 1.0)
	else
		frame:SetAlpha(0)
	end
end

-----------------------------------------------------------------------------

function Aloft:DetermineNameplateType(aloftData, healthBar)
	aloftData.originalHealthBarR, aloftData.originalHealthBarG, aloftData.originalHealthBarB, a = healthBar:GetStatusBarColor()
	local r, g, b = floor(aloftData.originalHealthBarR * 256), floor(aloftData.originalHealthBarG*256), floor(aloftData.originalHealthBarB*256)

	if r == 255 and g == 0 and b == 0 then
		aloftData.type = "hostile"
	elseif r == 0 and g == 0 and b == 255 then
		aloftData.type = "friendlyPlayer"
	elseif r == 0 and g == 255 and b == 0 then
		aloftData.type = "friendlyNPC"
	elseif r == 255 and g == 255 and b == 0 then
		aloftData.type = "neutral"
	else
--		ChatFrame3:AddMessage("rgb = "..r.."."..g.."."..b)
	end
end

-----------------------------------------------------------------------------

function Aloft:OnHealthBarValueChanged(aloftData)
	if aloftData.nameplateFrame:IsShown() then
		local healthBar = aloftData.healthBar

		aloftData.healthBarValue = healthBar:GetValue()
		_, aloftData.healthBarMaxValue = healthBar:GetMinMaxValues()
		if aloftData.nameTextFrameText ~= aloftData.nameTextFrame:AloftGetText() then
			self:OnNameplateShow(aloftData)
		else
			self:TriggerEvent("Aloft:OnHealthBarValueChanged", aloftData)
		end
	end
end

function Aloft:UpdateData(aloftData)
	for _,module in ipairs(dataSourceList) do
		module:UpdateData(aloftData)
	end
end

function Aloft:OnNameplateShow(aloftData)
	-- When a nameplate is first shown, grab some data about it
	local nameTextFrame = aloftData.nameTextFrame
	local levelTextFrame = aloftData.levelTextFrame
	local bossIconFrame = aloftData.bossIconFrame
	local healthBar = aloftData.healthBar

	if aloftData.nameTextFrameText ~= nameTextFrame:AloftGetText() then
		nameToNameplateMap[aloftData.name] = nil
		aloftData.name = nameTextFrame:AloftGetText()
		nameToNameplateMap[aloftData.name] = aloftData
		aloftData.nameTextFrameText = name
	else
		aloftData.nameTextFrameText = nameTextFrame:AloftGetText()
	end
	aloftData.healthBarValue = healthBar:GetValue()
	_, aloftData.healthBarMaxValue = healthBar:GetMinMaxValues()
	aloftData.isBoss = bossIconFrame:IsShown()
	aloftData.level = levelTextFrame:IsShown() and tonumber(levelTextFrame:GetText()) or -1

	self:DetermineNameplateType(aloftData, healthBar)
	self:UpdateData(aloftData)

	self:TriggerEvent("Aloft:OnNameplateShow", aloftData)
end

function Aloft:OnNameplateHide(aloftData)
	nameToNameplateMap[aloftData.name] = nil
	aloftData.name = "[invalid]"
	aloftData.nameTextFrameText = nil
	aloftData.type = "unknown"

	self:TriggerEvent("Aloft:OnNameplateHide", aloftData)
end

function Aloft:OnCastBarValueChanged(aloftData, castBar)
	if castBar:IsShown() then
		self:TriggerEvent("Aloft:OnCastBarValueChanged", aloftData)
	end
end

function Aloft:OnCastBarShow(aloftData, castBar)
	-- CastBar option, but handled in the core for logistics
	if self:AcquireDBNamespace("castBar").profile.hideExpired then
		local _, _, displayName = UnitCastingInfo("target")
		if not displayName then
			_, _, displayName = UnitChannelInfo("target")
			if not displayName then
				local castBarOverlayFrame = aloftData.castBarOverlayFrame
				local spellIconFrame = aloftData.spellIconFrame
				
				castBarOverlayFrame:Hide()
				castBarOverlayFrame:SetAlpha(0)

				spellIconFrame:Hide()
				spellIconFrame:SetAlpha(0)
				
				castBar:Hide()
				castBar:SetAlpha(0)
				return
			end
		end
	end

	self:TriggerEvent("Aloft:OnCastBarShow", aloftData)
end

function Aloft:OnCastBarHide(aloftData)
	self:TriggerEvent("Aloft:OnCastBarHide", aloftData)
end

-----------------------------------------------------------------------------

function Aloft:OnNameplateShowScript()
	visibleNameplateList[this.aloftData] = true
	Aloft:OnNameplateShow(this.aloftData)
	self.hooks[this]["OnShow"]()
end

function Aloft:OnNameplateHideScript()
	Aloft:OnNameplateHide(this.aloftData)
	visibleNameplateList[this.aloftData] = nil
	self.hooks[this]["OnHide"]()
end

function Aloft:OnHealthBarValueChangedScript()
	Aloft:OnHealthBarValueChanged(this.aloftData)
	self.hooks[this]["OnValueChanged"]()
end

function Aloft:OnCastBarValueChangedScript()
	Aloft:OnCastBarValueChanged(this.aloftData, this)
	self.hooks[this]["OnValueChanged"]()
end

function Aloft:OnCastBarShowScript()
	Aloft:OnCastBarShow(this.aloftData, this)
	self.hooks[this]["OnShow"]()
end

function Aloft:OnCastBarHideScript()
	Aloft:OnCastBarHide(this.aloftData)	
	self.hooks[this]["OnHide"]()
end

function Aloft:SetHealthBarColor(this, r, g, b, a)
	local aloftData = this.aloftData
	self.hooks[this].SetStatusBarColor(this, r, g, b, a)
	-- We read it back, because rounding seems to kick in and prevent them from being exactly what we sent..
	aloftData.healthBarR, aloftData.healthBarG, aloftData.healthBarB, aloftData.healthBarA = this:GetStatusBarColor()
end

function Aloft:SetNameTextFrameText(this, text)
	local aloftData = this.aloftData
	aloftData.nameTextFrameText = text
	self.hooks[this].SetText(this, text)
end

-----------------------------------------------------------------------------

local function AloftDataIsCurrentTarget(aloftData) return aloftData.nameplateFrame:GetAlpha() == 1 and aloftData.name == UnitName("target") end
local function AloftDataIsShown(aloftData) return aloftData.nameplateFrame:IsShown() end
local function AloftDataCreateFontString(aloftData) return aloftData.UIFrame:CreateFontString() end
local function AloftDataCreateTexture(aloftData) return aloftData.UIFrame:CreateTexture() end

local function AloftNameTextFrameGetText(nameTextFrame) if nameTextFrame.aloftData.name ~= "[invalid]" then return nameTextFrame.aloftData.name end return nameTextFrame:AloftGetText() end

function Aloft:SetupNameplate(frame)
	local aloftData = { type = "unknown", name = "[invalid]", IsCurrentTarget = AloftDataIsCurrentTarget, IsShown = AloftDataIsShown, CreateFontString = AloftDataCreateFontString, CreateTexture = AloftDataCreateTexture }
	frame.aloftData = aloftData
	nameplateList[aloftData] = true

	local overlayFrame, castBarOverlayFrame, spellIconFrame, highlightFrame, nameTextFrame, levelTextFrame, bossIconFrame, raidIconFrame = frame:GetRegions()
	local healthBar, castBar = frame:GetChildren()

	-- Reference all of these now, because we might dick with them in the Frame.lua module
	aloftData.healthBar = healthBar
	aloftData.castBar = castBar
	
	aloftData.overlayFrame = overlayFrame
	aloftData.castBarOverlayFrame = castBarOverlayFrame
	aloftData.spellIconFrame = spellIconFrame
	aloftData.highlightFrame = highlightFrame
	aloftData.nameTextFrame = nameTextFrame
	aloftData.levelTextFrame = levelTextFrame
	aloftData.bossIconFrame = bossIconFrame
	aloftData.raidIconFrame = raidIconFrame
	
	aloftData.nameplateFrame = frame
	aloftData.UIFrame = frame	

	-- Hook some stuff so that other addons don't spew when Aloft dicks with the frame's hierarchy
	-- Don't use AceHook as it will unhook these on module disable
	frame.AloftGetRegions = frame.GetRegions
	frame.GetNumRegions = function() return 8 end
	frame.GetRegions = function() return overlayFrame, castBarOverlayFrame, spellIconFrame, highlightFrame, nameTextFrame, levelTextFrame, bossIconFrame, raidIconFrame end
	frame.AloftGetChildren = frame.GetChildren
	frame.GetNumChildren = function() return 2 end
	frame.GetChildren = function() return healthBar, castBar end
	nameTextFrame.AloftGetText = nameTextFrame.GetText
	nameTextFrame.GetText = AloftNameTextFrameGetText
	healthBar.GetParent = function() return frame end
	castBar.GetParent = function() return frame end

	self:HookNameplate(aloftData)

	self:TriggerEvent("Aloft:SetupFrame", aloftData)

	self:OnNameplateShow(aloftData)
	if castBar:IsShown() then
		self:OnCastBarShow(aloftData, castBar)
	end
end

function Aloft:HookNameplate(aloftData)
	local nameplateFrame = aloftData.nameplateFrame
	self:HookScript(nameplateFrame, "OnShow", "OnNameplateShowScript")
	self:HookScript(nameplateFrame, "OnHide", "OnNameplateHideScript")

	local healthBar = aloftData.healthBar
	self:HookScript(healthBar, "OnValueChanged", "OnHealthBarValueChangedScript")
	self:Hook(healthBar, "SetStatusBarColor", "SetHealthBarColor", true)
	healthBar.aloftData = aloftData

	local castBar = aloftData.castBar
	self:HookScript(castBar, "OnShow", "OnCastBarShowScript")
	self:HookScript(castBar, "OnHide", "OnCastBarHideScript")
	self:HookScript(castBar, "OnValueChanged", "OnCastBarValueChangedScript")
	castBar.aloftData = aloftData

	local nameTextFrame = aloftData.nameTextFrame
	self:Hook(nameTextFrame, "SetText", "SetNameTextFrameText", true)
	nameTextFrame.aloftData = aloftData
end

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------

Aloft.modulePrototype.PrepareText = Aloft.PrepareText
Aloft.modulePrototype.PlaceFrame = Aloft.PlaceFrame

function Aloft.modulePrototype:RequiresData()
	-- eg. return "class", "isPetName", "isGuildMember"
end

function Aloft.modulePrototype:ProvidesData()
end

function Aloft.modulePrototype:EnableDataSource()
end

function Aloft.modulePrototype:IsDataAvailable(name)
	return dataNameToModuleMap[name] ~= nil
end

function Aloft.modulePrototype:IsPurelyData()
	return self:ProvidesData() ~= nil
end

function Aloft.modulePrototype:RemoveAloftData(name)
	for aloftData in Aloft:IterateNameplates() do
		aloftData[name] = nil
	end
end

function Aloft.modulePrototype:RemoveProvidedData(...)
	for i=1,select('#',...) do
		self:RemoveAloftData(select(i, ...))
	end
end

function Aloft.modulePrototype:DisableDataSource()
	local dataName = self:ProvidesData()
	if dataName then
		self:UnregisterAllEvents()
		self:RemoveProvidedData()
	end
end

function Aloft.modulePrototype:UpdateData(aloftData)
	DEFAULT_CHAT_FRAME:AddMessage("DataSource's UpdateData not overriden.")
end

function Aloft.modulePrototype:CancelEvent(eventName)
	if self:IsEventRegistered(eventName) then
		self:UnregisterEvent(eventName)
	end
end

function Aloft.modulePrototype:IsArrayEqual(a, b)
	if #a ~= #b then return false end
	for i=1,#a do
		if a[i] ~= b[i] then return false end
	end
	return true
end

-- Provide these functions so that the log doesn't fill up
function Aloft:RedisplayNameplates()
	if NAMEPLATES_ON then
		Aloft.HideNameplates()
		self:ScheduleEvent("AloftRedisplayNameplates", Aloft.ShowNameplates, 0.1)
	end
end

function Aloft:RedisplayFriendlyNameplates()
	if FRIENDNAMEPLATES_ON then
		Aloft.HideFriendNameplates()
		self:ScheduleEvent("AloftRedisplayFriendlyNameplate", Aloft.ShowFriendNameplates, 0.1)
	end
end

function Aloft:FormatLargeValue(value)
	if value < 10000 then
		-- Force it to an integer
		return string.format("%d", value)
	elseif value < 100000 then
		return string.format("%.1fk", value / 1000)
	elseif value < 1000000 then
		return string.format("%dk", value / 1000)
	else
		return string.format("%.2fm", value / 1000000)
	end
end

-----------------------------------------------------------------------------
