-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftGuildText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Guild Text"] = true,
	["Guild text options"] = true,
	["Enable"] = true,
	["Show guild information on nameplates"] = true,
	
	["Attach to Name"] = true,
	["Attaches guild text to name"] = true,
	["Attach Format"] = true,
	["Selects the text format to use"] = true,
	["Name - Guild"] = true,
	["Name (Guild)"]  = true,

	["Short"] = true,
	["Use short guild names"] = true,
	["Set Short Name"] = true,
	["<Any String>"] = true,
	["Sets the short name for your current target"] = true,
	["Show Own"] = true,
	["Show the guild names of your own guild members"] = true,

	["Font"] = true,
	["Sets the font for guild text"] = true,
	["Font Size"] = true,
	["Sets the font height of the guild text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on guild text"] = true,
	["Outline"] = true,
	["Sets the outline for guild text"] = true,
	["Anchor"] = true,
	["Sets the anchor for the guild text"] = true,
	["Anchor To"] = true,
	["Sets the relative point on the health bar to anchor the guild text"] = true,
	["X Offset"] = true,
	["X offset of the guild text"] = true,
	["Y Offset"] = true,
	["Y offset of the guild text"] = true,
	["Alpha"] = true,
	["Sets the alpha of the guild text"] = true,
	["Color"] = true,
	["Sets the guild text color"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
	
	["NoSetShortName"] = "|cffff0000You must have a target to set the short name|r",
	["ShortNameAdded"] = "Added |cff00ff00%s|r -> |cff00ff00%s|r",
} end)

-----------------------------------------------------------------------------

local myGuild = nil

local attachFormats =
{
	["NONE"]		= { "[name]", "[name]", "[name]", "[name]" },
	["HYPHEN"]		= { "[name][guild:\" - \":hideGuild][guild:hideGuild]", "[name][shortGuild:\" - \":hideGuild][shortGuild:hideGuild]", "[name][guild:\" - \"][guild]", "[name][shortGuild:\" - \"][shortGuild]" },
	["BRACKETS"]	= { "[name] [guild:hideGuild:bracket]", "[name] [shortGuild:hideGuild:bracket]", "[name] [guild:bracket]", "[name] [shortGuild:bracket]" }
}

-----------------------------------------------------------------------------

local AloftGuildText = Aloft:NewModule("GuildText", "AceEvent-2.0", "AceConsole-2.0")
AloftGuildText.db = Aloft:AcquireDBNamespace("guildText")
Aloft:RegisterDefaults("guildText", "profile", {
	enable			= false,
	useShort		= true,
	showOwn			= true,
	attachToName	= true,
	attachFormat	= "HYPHEN",
	alpha			= 1,
	point			= "BOTTOMLEFT",
	relativeToPoint	= "TOPLEFT",
	offsetX			= 0,
	offsetY			= 0,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= false,
	outline			= "OUTLINE",
	color			= { 1, 1, 1 }
})

Aloft.Options.args.guildText =
{
	type = 'group',
	name = L["Guild Text"],
	desc = L["Guild text options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show guild information on nameplates"],
			get = function() return AloftGuildText.db.profile.enable end,
			set = function(v) AloftGuildText.db.profile.enable = v AloftGuildText:UpdateAttachedText() Aloft:DetermineDataSources() AloftGuildText:UpdateAll() end,
			order = 1
		},
		attachToName =
		{
			type = 'toggle',
			name = L["Attach to Name"],
			desc = L["Attaches guild text to name"],
			disabled = function() return not AloftGuildText.db.profile.enable end,
			get = function() return AloftGuildText.db.profile.attachToName end,
			set = function(v) AloftGuildText.db.profile.attachToName = v AloftGuildText:UpdateAttachedText() Aloft:DetermineDataSources() AloftGuildText:UpdateAll() end,
			order = 2
		},
		attachFormat =
		{
			type = 'text',
			name = L["Attach Format"],
			desc = L["Selects the text format to use"],
			disabled = function() return not AloftGuildText.db.profile.enable or not AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.attachFormat end,
			set = function(v) AloftGuildText.db.profile.attachFormat = v AloftGuildText:UpdateAttachedText() Aloft:DetermineDataSources() AloftGuildText:UpdateAll() end,
			validate = { ["HYPHEN"] = L["Name - Guild"], ["BRACKETS"] = L["Name (Guild)"] },
			order = 3
		},
		short =
		{
			type = 'toggle',
			name = L["Short"],
			desc = L["Use short guild names"],
			disabled = function() return not AloftGuildText.db.profile.enable end,
			get = function() return AloftGuildText.db.profile.useShort end,
			set = function(v) AloftGuildText.db.profile.useShort = v AloftGuildText:UpdateAttachedText() Aloft:DetermineDataSources() AloftGuildText:UpdateAll() end,
			order = 4
		},
		setShort =
		{
			type = 'text',
			name = L["Set Short Name"],
			desc = L["Sets the short name for your current target"],
			usage = L["<Any String>"],
			disabled = function() return not AloftGuildText.db.profile.enable or not AloftGuildText.db.profile.useShort end,
			get = function() return "" end,
			set = function(v) 
					if UnitExists("target") then
						local targetGuild = GetGuildInfo("target") or ""
						Aloft:GetModule("ShortGuildData"):AddData(targetGuild, v)
						AloftGuildText:Print(string.format(L["ShortNameAdded"], targetGuild, v))
					else
						AloftGuildText:Print(L["NoSetShortName"])
					end
				  end,
			order = 5
		},
		showOwn =
		{
			type = 'toggle',
			name = L["Show Own"],
			desc = L["Show the guild names of your own guild members"],
			disabled = function() return not AloftGuildText.db.profile.enable end,
			get = function() return AloftGuildText.db.profile.showOwn end,
			set = function(v) AloftGuildText.db.profile.showOwn = v AloftGuildText:UpdateAttachedText() Aloft:DetermineDataSources() AloftGuildText:UpdateAll() end,
			order = 6
		},
		font =
		{
			type = "text", 
			name = L["Font"], 
			desc = L["Sets the font for guild text"],
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.font end,
			set = function(v) AloftGuildText.db.profile.font = v AloftGuildText:UpdateAll() end,
			validate = SML_font,
		},
		fontSize =
		{
			type = 'range',
			name = L["Font Size"],
			desc = L["Sets the font height of the guild text"],
			max = 16,
			min = 5,
			step = 1,
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.fontSize end,
			set = function(value) AloftGuildText.db.profile.fontSize = value AloftGuildText:UpdateAll() end
		},
		shadow =
		{
			type = 'toggle',
			name = L["Font Shadow"],
			desc = L["Show font shadow on guild text"],
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.shadow end,
			set = function(v) AloftGuildText.db.profile.shadow = v AloftGuildText:UpdateAll() end
		},
		outline =
		{
			type = 'text',
			name = L["Outline"],
			desc = L["Sets the outline for guild text"],
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.outline end,
			set = function(value) AloftGuildText.db.profile.outline = value AloftGuildText:UpdateAll() end,
			validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
		},
		anchor = {
			type = "text", 
			name = L["Anchor"], 
			desc = L["Sets the anchor for the guild text"],
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.point end,
			set = function(v) AloftGuildText.db.profile.point = v AloftGuildText:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		anchorto = {
			type = "text", 
			name = L["Anchor To"], 
			desc = L["Sets the relative point on the health bar to anchor the guild text"],
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.relativeToPoint end,
			set = function(v) AloftGuildText.db.profile.relativeToPoint = v AloftGuildText:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		offsetX =
		{
			type = 'range',
			name = L["X Offset"],
			desc = L["X offset of the guild text"],
			min = -32,
			max = 32,
			step = 1,
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.offsetX end,
			set = function(v) AloftGuildText.db.profile.offsetX = v AloftGuildText:UpdateAll() end
		},
		offsetY =
		{
			type = 'range',
			name = L["Y Offset"],
			desc = L["Y offset of the guild text"],
			min = -32,
			max = 32,
			step = 1,
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.offsetY end,
			set = function(v) AloftGuildText.db.profile.offsetY = v AloftGuildText:UpdateAll() end
		},
		alpha =
		{
			type = 'range',
			name = L["Alpha"],
			desc = L["Sets the alpha of the guild text"],
			min = 0,
			max = 1.0,
			step = 0.05,
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return AloftGuildText.db.profile.alpha end,
			set = function(v) AloftGuildText.db.profile.alpha = v AloftGuildText:UpdateAll() end
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the guild text color"],
			disabled = function() return not AloftGuildText.db.profile.enable or AloftGuildText.db.profile.attachToName end,
			get = function() return unpack(AloftGuildText.db.profile.color) end,
			set = function(r, g, b, a) AloftGuildText.db.profile.color = { r, g, b, a } AloftGuildText:UpdateAll() end,
		},
	},
}

-----------------------------------------------------------------------------

function AloftGuildText:UpdateAll()
	self:RegisterEvents()

	for aloftData in Aloft:IterateNameplates() do
		if self.db.profile.enable then
			if self.db.profile.attachToName then
				local guildTextFrame = aloftData.guildTextFrame
				if guildTextFrame then
					guildTextFrame:Hide()
				end
			else
				self:SetupFrame(aloftData)
				
				if aloftData:IsShown() then
					self:UpdateText(aloftData)
				end
			end
		else
			local guildTextFrame = aloftData.guildTextFrame
			if guildTextFrame then
				guildTextFrame:Hide()
			end
		end
	end	
	Aloft:GetModule("NameText"):UpdateAll()
end

function AloftGuildText:UpdateAttachedText()
	local nameTextModule = Aloft:GetModule("NameText")
	
	local attachFormatIndex = 1
	if self.db.profile.useShort then attachFormatIndex = 2 end
	if self.db.profile.showOwn then attachFormatIndex = attachFormatIndex+2 end

	local attachFormat = self.db.profile.attachFormat
	if not self.db.profile.enable or not self.db.profile.attachToName then
		attachFormat = "NONE"
	end
	
	ChatFrame3:AddMessage("Using index: "..attachFormatIndex)
	nameTextModule.db.profile.format = attachFormats[attachFormat][attachFormatIndex]
end

-----------------------------------------------------------------------------

function AloftGuildText:RegisterEvents()
	self:CancelEvent("Aloft:SetupFrame")
	self:CancelEvent("Aloft:OnNameplateShow")
	self:CancelEvent("Aloft:OnGuildDataChanged")
	
	if self.db.profile.enable then
		if not self.db.profile.attachToName then
			self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
			self:RegisterEvent("Aloft:OnNameplateShow", "UpdateText")
			self:RegisterEvent("Aloft:OnGuildDataChanged", "UpdateText")
		end
		
		if IsInGuild() then
			myGuild = GetGuildInfo("player")
		else
			myGuild = nil
		end
	end
end

function AloftGuildText:OnEnable()
	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:UpdateAll()
		end
	end)

	self:UpdateAll()
end

function AloftGuildText:OnDisable()
	self:UnregisterAllEvents()
	
	for aloftData in Aloft:IterateNameplates() do
		local guildTextFrame = aloftData.guildTextFrame
		if guildTextFrame then
			guildTextFrame:Hide()
		end
	end
end

function AloftGuildText:RequiresData()
	local profile = self.db.profile
	if profile.enable then
		if profile.useShort then
			return "shortGuild"
		else
			return "guild"
		end
	end
end

-----------------------------------------------------------------------------

function AloftGuildText:SetupFrame(aloftData)
	local healthBar = aloftData.healthBar
	local guildTextFrame = aloftData.guildTextFrame
	local profile = self.db.profile
	
	if guildTextFrame then
		guildTextFrame:Show()
	else
		guildTextFrame = aloftData:CreateFontString()
		aloftData.guildTextFrame = guildTextFrame
	end

	self:PrepareText(guildTextFrame, profile)
	self:PlaceFrame(guildTextFrame, aloftData.healthBar, self.db.profile)
end

function AloftGuildText:UpdateText(aloftData)
	local guildTextFrame = aloftData.guildTextFrame
	local profile = self.db.profile

	local guildText = profile.useShort and aloftData.shortGuild or aloftData.guild
	if not profile.showOwn and aloftData.guild == myGuild then
		guildText = nil
	end

	if guildText then
		guildTextFrame:SetText(guildText)
		guildTextFrame:Show()
	else
		guildTextFrame:Hide()
	end
end

-----------------------------------------------------------------------------
